@@ -523,7 +523,21 @@ dectalk_synthesis_driver_finalize(GObject *obj)
 	DectalkSynthesisDriver *d = DECTALK_SYNTHESIS_DRIVER(obj);
 	
 	if (d->handle != NULL) {
+#if 0
+	  /* [[[WDW: Commented this out for gnome-speech 0.3.10 as
+	     a workaround for the problem where DECtalk can suddenly
+	     go silent and we cannot detect it.  When we try to exit
+	     the driver gracefully afterwards, the driver will hang
+	     on this call.  The impact of commenting this out is that
+	     any speech in progress will stop when the driver exits.
+	     Oh well.  At least it will exit and not hang.  Note that
+	     we will also be working with Fonix to determine the
+	     underlying cause of why DECtalk is suddenly going silent.
+	     If we can solve that problem, this line probably can be
+	     re-added.
+	  */
 		TextToSpeechSync (d->handle);
+#endif
 		TextToSpeechShutdown(d->handle);
 	}
 	if (d->timeout_id > 0)
@@ -666,7 +680,7 @@ dectalk_synthesis_driver_wait (DectalkSynthesisDriver *d)
 gboolean
 dectalk_synthesis_driver_is_speaking (DectalkSynthesisDriver *d)
 {
-	gint lparam1, lparam2;
+	DWORD lparam1, lparam2;
 
 	TextToSpeechGetStatus (d->handle, &lparam1, &lparam2, STATUS_SPEAKING);
 	return TRUE;
