@@ -313,7 +313,6 @@ festival_response_sock (GIOChannel   *source,
 #ifdef FESTIVAL_DEBUG_CONTROL
     	    fprintf (stderr, "\nRECEIVE QUERY");
 #endif
-	    driver->is_querying = FALSE;
         }
 	else if (ack && strncmp (ack, "(VOICES:", 8) == 0)
 	{
@@ -483,11 +482,11 @@ festival_start (FestivalSynthesisDriver *d)
 	    return FALSE;
 	
         d->channel_pipe = g_io_channel_unix_new(d->pipe);
-	g_io_channel_set_encoding (d->channel_pipe, "ISO-8859-1", NULL);
+	g_io_channel_set_encoding (d->channel_pipe, "UTF-8", NULL);
         g_io_add_watch (d->channel_pipe, G_IO_IN , festival_response_pipe, d);
 	
 	d->channel_sock = g_io_channel_unix_new(d->sock);
-	g_io_channel_set_encoding (d->channel_sock, "ISO-8859-1", NULL);
+	g_io_channel_set_encoding (d->channel_sock, "UTF-8", NULL);
         g_io_add_watch (d->channel_sock, G_IO_IN, festival_response_sock, d);
 
 	if (festival_server_exists) {
@@ -793,7 +792,6 @@ festival_synthesis_driver_init (FestivalSynthesisDriver *d)
         d->crt_clbs = NULL;
         d->is_shutting_up = FALSE;
 	d->is_speaking = FALSE;
-        d->is_querying = FALSE;
         d->list = NULL;
         d->crt_id = 0;
         d->queue_length = 0;	
@@ -848,6 +846,7 @@ void
 festival_synthesis_driver_say_raw (FestivalSynthesisDriver *d,
 			    	   gchar 		   *text)
 {
+	GIOStatus status;
         int l, written;
 
 	g_assert (text && IS_FESTIVAL_SYNTHESIS_DRIVER (d) && d->channel_sock);
@@ -855,8 +854,16 @@ festival_synthesis_driver_say_raw (FestivalSynthesisDriver *d,
         fprintf (stderr, "\nRAW SEND :\"%s\"", text);
 #endif
 	l = strlen (text);
-        g_io_channel_write_chars (d->channel_sock, text, l, &written, NULL);
+        status = g_io_channel_write_chars (d->channel_sock, text, l, &written, NULL);
         g_io_channel_flush (d->channel_sock, NULL);
+	if (status == G_IO_STATUS_ERROR)
+	{
+	    kill (d->pid, 9);
+	    fprintf (stderr, 
+		     "g_ui_channel_write_chars gave G_IO_STATUE_ERROR\n");
+	    fprintf (stderr, "ABORTING.\n");
+	    exit (1);
+	}
 }
 
 
@@ -887,14 +894,10 @@ festival_querying_queue (gpointer data)
 
 	g_assert (IS_FESTIVAL_SYNTHESIS_DRIVER (driver));
 
-        if (!driver->is_querying && !driver->is_shutting_up && driver->is_speaking)
-	{	
 #ifdef FESTIVAL_DEBUG_CONTROL
-	    fprintf (stderr, "\nSENT QUERY");
+	fprintf (stderr, "\nSENT QUERY");
 #endif
-	    festival_synthesis_driver_say_raw (driver, "(audio_mode 'query)\n");
-	    driver->is_querying = TRUE;
-	}
+	festival_synthesis_driver_say_raw (driver, "(audio_mode 'query)\n");
     
         return TRUE;
 }
