@@ -45,6 +45,53 @@ typedef struct {
 static gint text_id = 0;
 static GObjectClass *parent_class;
 
+typedef struct _eciLocale {
+	char *pLocale;
+	enum ECILanguageDialect langID;
+} eciLocale, *eciLocaleList;
+
+static eciLocale eciLocales[] = {
+	{"en_US", eciGeneralAmericanEnglish },
+	{"en_GB", eciBritishEnglish },
+	{"es_ES", eciCastilianSpanish},
+	{"es_MX", eciMexicanSpanish },
+	{"fr_FR", eciStandardFrench },
+	{"ca_FR", eciCanadianFrench },
+	{"de_DE", eciStandardGerman },
+	{"it_IT", eciStandardItalian},
+	{"zh_CN", eciMandarinChinese},
+	{"zh_CN", eciMandarinChineseGB },
+	{"zh_CN", eciMandarinChinesePinYin},
+	{"zh_CN", eciMandarinChineseUCS},
+	{"zh_TW", eciTaiwaneseMandarin },
+	{"zh_TW", eciTaiwaneseMandarinBig5},
+	{"zh_TW", eciTaiwaneseMandarinZhuYin},
+	{"zh_TW", eciTaiwaneseMandarinPinYin},
+	{"zh_TW", eciTaiwaneseMandarinUCS },
+	{"pt_BR", eciBrazilianPortuguese},
+	{"ja_JP", eciStandardJapanese},
+	{"ja_JP", eciStandardJapaneseSJIS },
+	{"ja_JP", eciStandardJapaneseUCS},
+	{"fi_FI", eciStandardFinnish},
+	{"ko_KR", eciStandardKorean },
+	{"ko_KR", eciStandardKoreanUHC },
+	{"ko_KR", eciStandardKoreanUCS },
+	{"zh_HK", eciStandardCantonese },
+	{"zh_HK", eciStandardCantoneseGB},
+	{"zh_HK", eciStandardCantoneseUCS },
+	{"zh_HK", eciHongKongCantonese },
+	{"zh_HK", eciHongKongCantoneseBig5},
+	{"zh_HK", eciHongKongCantoneseUCS },
+	{"nl_BE", eciStandardDutch},
+	{"no_NO", eciStandardNorwegian },
+	{"sv_SE", eciStandardSwedish},
+	{"da_DK", eciStandardDanish },
+	{"en_US", eciStandardReserved},
+	{"th_TH", eciStandardThai },
+	{"th_TH", eciStandardThaiTIS},
+	{NULL, 0}
+};
+
 /*
  *
  * index_queue_entry functions
@@ -70,12 +117,14 @@ viavoice_synthesis_driver_flush_index_queue (ViavoiceSynthesisDriver *d)
 	
 	/* Flush the index queues */
 
+	g_mutex_lock (d->mutex);
 	for (tmp = d->index_queue; tmp; tmp = tmp->next) {
 		index_queue_entry *e = (index_queue_entry *) tmp->data;
 		index_queue_entry_destroy (e);
 	}
 	g_slist_free (d->index_queue);
 	d->index_queue = NULL;
+	g_mutex_unlock (d->mutex);
 }
 
 static ViavoiceSynthesisDriver *
@@ -96,9 +145,9 @@ viavoice_synthesis_driver_timeout_callback (void *data)
 
 
 
-static ECICallbackReturn
+static enum ECICallbackReturn
 viavoice_synthesis_driver_index_callback (ECIHand handle,
-			   ECIMessage msg,
+			   enum ECIMessage msg,
 			   long param,
 			   void *data)
 {
@@ -106,10 +155,12 @@ viavoice_synthesis_driver_index_callback (ECIHand handle,
 	index_queue_entry *e = NULL;
 	CORBA_Environment ev;
 	
+	g_mutex_lock (d->mutex);
 	if (d && d->index_queue) {
 		e = (index_queue_entry *) d->index_queue->data;
 		d->index_queue = g_slist_remove_link (d->index_queue, d->index_queue);
 	}
+	g_mutex_unlock (d->mutex);
 	switch (msg) {
 	case eciIndexReply:
 		if (e) {
@@ -233,33 +284,49 @@ viavoice_isInitialized (PortableServer_Servant servant,
 
 
 static GSList *
-get_voice_list (void)
+get_voice_list (GNOME_Speech_VoiceInfo *vs)
 {
 	GSList *l = NULL;
 	ECIHand tmp_handle;
-	int i;
+	int h, i;
+	enum ECILanguageDialect origLang;
   
 	tmp_handle = eciNew ();
 	if (tmp_handle == NULL_ECI_HAND)
 		return NULL;
 
-	for (i = 1; i <= ECI_PRESET_VOICES; i++) {
-		GNOME_Speech_VoiceInfo *info;
-		char tmp[ECI_VOICE_NAME_LENGTH + 1];
-		int gender;
-    
-		info = GNOME_Speech_VoiceInfo__alloc ();
-		eciGetVoiceName (tmp_handle, i, tmp);
-		info->language = CORBA_string_dup("en_US");
-		info->name = CORBA_string_dup (tmp);
-		gender = eciGetVoiceParam (tmp_handle, i, eciGender);
-		if (gender == 0)
-			info->gender = GNOME_Speech_gender_male;
-		else if (gender == 1)
-			info->gender = GNOME_Speech_gender_female;
-		l = g_slist_prepend (l, info);
+	origLang = eciGetParam(tmp_handle, eciLanguageDialect);
+
+	for (h = 0; NULL != eciLocales[h].pLocale; h++) {
+
+		if (vs && (0 != strcmp(vs->language, eciLocales[h].pLocale))) {
+			continue;
+		}
+
+		if (eciSetParam(tmp_handle, eciLanguageDialect,
+		eciLocales[h].langID) < 0) {
+			continue;
+		}
+
+		for (i = 1; i <= ECI_PRESET_VOICES; i++) {
+			GNOME_Speech_VoiceInfo *info;
+			char tmp[ECI_VOICE_NAME_LENGTH + 1];
+			int gender;
+	    
+			info = GNOME_Speech_VoiceInfo__alloc ();
+			eciGetVoiceName (tmp_handle, i, tmp);
+			info->language= CORBA_string_dup(eciLocales[h].pLocale);
+			info->name = CORBA_string_dup (tmp);
+			gender = eciGetVoiceParam (tmp_handle, i, eciGender);
+			if (gender == 0)
+				info->gender = GNOME_Speech_gender_male;
+			else if (gender == 1)
+				info->gender = GNOME_Speech_gender_female;
+			l = g_slist_prepend (l, info);
+		}
 	}
 	l = g_slist_reverse (l);
+	eciSetParam(tmp_handle, eciLanguageDialect, origLang);
 	eciDelete (tmp_handle);
 	return l;
 }
@@ -317,7 +384,7 @@ viavoice_getVoices (PortableServer_Servant servant,
 	GNOME_Speech_VoiceInfoList *rv;
 	GSList *l;
 
-	l = get_voice_list ();
+	l = get_voice_list (voice_spec);
 	rv = voice_info_list_from_voice_list (l);
 	voice_list_free (l);
 	return rv;
@@ -332,7 +399,7 @@ viavoice_getAllVoices (PortableServer_Servant servant,
 	GNOME_Speech_VoiceInfoList *rv;
 	GSList *l;
 
-	l = get_voice_list ();
+	l = get_voice_list (NULL);
 	rv = voice_info_list_from_voice_list (l);
 	voice_list_free (l);
 	return rv;
@@ -355,6 +422,7 @@ viavoice_createSpeaker (PortableServer_Servant servant,
 static void
 viavoice_synthesis_driver_init (ViavoiceSynthesisDriver *driver)
 {
+	driver->mutex = g_mutex_new ();
 	driver->last_speaker = NULL;
 	driver->index_queue = NULL;
 	driver->timeout_id = -1;
@@ -444,6 +512,11 @@ main (int argc,
 		g_error ("Could not initialize Bonobo Activation / Bonobo");
 	}
 
+	/* If threads haven't been initialized, initialize them */
+
+	if (!g_threads_got_initialized)
+		g_thread_init (NULL);
+	
 	obj_id = "OAFIID:GNOME_Speech_SynthesisDriver_Viavoice:proto0.3";
 
 	driver = viavoice_synthesis_driver_new ();
@@ -480,7 +553,9 @@ viavoice_synthesis_driver_add_index (ViavoiceSynthesisDriver *d,
 	e->type = type;
 	e->text_id = text_id;
 	e->offset = offset;
+	g_mutex_lock (d->mutex);
 	d->index_queue = g_slist_append(d->index_queue, e);
+	g_mutex_unlock (d->mutex);
 
 	eciInsertIndex (d->handle, 0);
 }
@@ -566,12 +641,20 @@ viavoice_synthesis_driver_is_speaking (ViavoiceSynthesisDriver *d)
 
 void
 viavoice_synthesis_driver_set_voice_param (const ViavoiceSynthesisDriver *d,
-					   ECIVoiceParam param,
+					   enum ECIVoiceParam param,
 					   gint new_value)
 {
 	eciSetVoiceParam (d->handle, 0, param, new_value);
 }
 
+void
+viavoice_synthesis_driver_set_device_param (const ViavoiceSynthesisDriver *d,
+					   enum ECIParam param,
+					   gint new_value)
+{
+	eciSetParam (d->handle, param, new_value);
+}
+
 
 gboolean
 viavoice_synthesis_driver_set_voice (const ViavoiceSynthesisDriver *d,
@@ -581,3 +664,29 @@ viavoice_synthesis_driver_set_voice (const ViavoiceSynthesisDriver *d,
 		      (int) v,
 		      0);
 }
+
+char *
+dialect_to_language(enum ECILanguageDialect langID) {
+	int h;
+
+	for (h = 0; NULL != eciLocales[h].pLocale; h++) {
+
+		if (langID == eciLocales[h].langID) {
+			return  eciLocales[h].pLocale;
+		}
+	}
+}
+
+enum ECILanguageDialect
+language_to_dialect(char *lang) {
+	int h;
+
+	for (h = 0; NULL != eciLocales[h].pLocale; h++) {
+
+		if (0 != strcmp(lang, eciLocales[h].pLocale)) {
+			return eciLocales[h].langID;
+		}
+	}
+
+	return eciGeneralAmericanEnglish;
+}
